@@ -531,13 +531,13 @@ list_printers (int mode, int reg_type_no, int isFax)
       if (errno != EAGAIN && errno != EINTR)
       {
 	perror("ERROR: Unable to read ippfind output");
-	exit_status = -1;
+	exit_status = 1;
 	goto error;
       }
     }
   } else {
     perror("ERROR: Unable to open ippfind output data stream");
-    exit_status = -1;
+    exit_status = 1;
     goto error;
   }
 
@@ -572,6 +572,10 @@ list_printers (int mode, int reg_type_no, int isFax)
   if (WIFEXITED(wait_status)) {
     /* Via exit() anywhere or return() in the main() function */
     exit_status = WEXITSTATUS(wait_status);
+    /* if we get 1 from ippfind, it is actually a correct value, not an error,
+     * because CUPS backends return 0 if they don't find any queues */
+    if (exit_status == 1)
+      exit_status = 0;
     if (exit_status)
       fprintf(stderr, "ERROR: ippfind (PID %d) stopped with status %d!\n",
 	      ippfind_pid, exit_status);
